@@ -11,8 +11,8 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with this library; see the file COPYING.LIB.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+// Boston, MA 02110-1301, USA.
 
 #ifndef LIBATLANTIC_TRADE_H
 #define LIBATLANTIC_TRADE_H
@@ -22,6 +22,7 @@
 #include <qptrlist.h>
 
 #include "libatlantic_export.h"
+#include "player.h"
 
 class Player;
 class Trade;
@@ -113,26 +114,6 @@ public:
 
 	bool isRejected() { return m_rejected; }
 	
-	/**
-	 * select the Trade of the given template type, with the given
-	 * from and to
-	 **/
-	template <class Type> Type *select(Player *from, Player *to)
-	{
-		for (QPtrListIterator<Player> i(mPlayers); *i; ++i)
-		{
-			if (
-					i.current()->from()==from
-					&& i.current()->to()==to
-					&& dynamic_cast<Player*>(*i)
-				)
-			{
-				return *i;
-			}
-		}
-	
-	}
-
 private slots:	
 	/**
 	 * tell someone that this changed
