@@ -18,7 +18,7 @@
 #endif
 
 
-static const char *description =
+static const char description[] =
 	I18N_NOOP("KDE Space Game");
 
 int main( int argc, char *argv[] )
