@@ -37,7 +37,7 @@
 
 #define PI_X_2                  6.283185307
 
-struct
+static struct
 {
     int id;
     const char *path;
