@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <kapplication.h>
@@ -26,6 +26,9 @@
 
 #include "kbounce.h"
 
+#include <khighscore.h>
+#include <highscores.h>
+
 using namespace std;
 
 #ifdef HAVE_ARTS
@@ -37,6 +40,8 @@ static const char version[] = "0.5";
 
 int main(int argc, char **argv)
 {
+  KHighscore::init("kbounce");
+
   KAboutData aboutData( "kbounce", I18N_NOOP("KBounce"),
     version, description, KAboutData::License_GPL,
     "(c) 2000, Stefan Schimanski");
@@ -51,6 +56,8 @@ int main(int argc, char **argv)
   KApplication a;
   KGlobal::locale()->insertCatalogue("libkdegames");
 
+  ExtManager manager;
+
   // setup MCOP
 #ifdef HAVE_ARTS
   Dispatcher dispatcher;
