@@ -12,7 +12,8 @@ class FEField : public Field
     FEField(QWidget *parent);
 
  private slots:
-    void removedUpdated(uint chained, uint removed);
+    virtual void removedUpdated();
+    void settingsChanged();
 };
 
 #endif
