@@ -1,6 +1,4 @@
 /***************************************************************************
-                          cell.h  -  description
-                             -------------------
     begin                : Fri May 19 2000
     copyright            : (C) 2000 by Roman Razilov
     email                : Roman.Razilov@gmx.de
@@ -14,10 +12,11 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-#ifndef __cell_h
-#define __cell_h
-#include "cfg.h"
 
+#ifndef cell_h
+#define cell_h
+
+#include "cfg.h"
 
 // enum int AnimType {ANIM_NO,ANIM_BORN,ANIM_JUMP,ANIM_RUN,ANIM_BURN,ANIM_BLOCK};
 
