@@ -6,6 +6,7 @@
 #include <kpushbutton.h>
 #include <kstandarddirs.h>
 #include <kstdguiitem.h>
+#include <kimageio.h>
 
 #include <qcombobox.h>
 #include <qhgroupbox.h>
@@ -32,7 +33,7 @@ Preview::Preview(QWidget* parent) : KDialogBase(parent), tiles(true)
 	combo = new QComboBox(false, group);
 	connect(combo, SIGNAL(activated(int)), SLOT(selectionChanged(int)));
 
-	loadButton = new KPushButton(i18n("Load"), group);
+	loadButton = new KPushButton(i18n("Load..."), group);
 	connect( loadButton, SIGNAL(clicked()), SLOT(load()) );
 	
 	drawFrame = new FrameImage(page);
@@ -73,12 +74,9 @@ void Preview::markUnchanged()
 
 void Preview::initialise(const PreviewType type, const char *extension)
 {
-	QString tile = "pics/" + Prefs::tileSet();
-	QString back = "pics/" + Prefs::background();
-	QString layout = "pics/" + Prefs::layout();
-	tile = locate("appdata", tile);
-	back = locate("appdata", back);
-	layout = locate("appdata", layout);
+	QString tile = Prefs::tileSet();
+	QString back = Prefs::background();
+	QString layout = Prefs::layout();
 
 	// set up the concept of the current file. Initialised to the preferences
 	// value initially. Set the caption to indicate what we are doing
@@ -87,24 +85,24 @@ void Preview::initialise(const PreviewType type, const char *extension)
 		case background:
 			setCaption(i18n("Change Background Image"));
 			selectedFile = back;
-			fileSelector = i18n("*.bgnd|Background Image\n"
-				     "*.bmp|Windows Bitmap File (*.bmp)\n");
+			fileSelector = i18n("*.bgnd|Background Image (*.bgnd)\n");
+			fileSelector += KImageIO::pattern()+"\n";
 		break;
 		
 		case tileset:
 			setCaption(i18n("Change Tile Set"));
-			fileSelector = i18n("*.tileset|Tile Set File\n");
+			fileSelector = i18n("*.tileset|Tile Set File (*.tileset)\n");
 			selectedFile = tile;
 		break;
 		
 		case board:
-			fileSelector = i18n("*.layout|Board Layout File\n");
+			fileSelector = i18n("*.layout|Board Layout File (*.layout)\n");
 			setCaption(i18n("Change Board Layout"));
 			selectedFile = layout;
 		break;
 		
 		case theme:
-			fileSelector = i18n("*.theme|KMahjongg Theme\n");
+			fileSelector = i18n("*.theme|KMahjongg Theme File (*.theme)\n");
 			setCaption(i18n("Choose Theme"));
 			selectedFile="";
 			
@@ -116,7 +114,7 @@ void Preview::initialise(const PreviewType type, const char *extension)
 		break;
 	}
 	
-	fileSelector += i18n("*|All Files\n");
+	fileSelector += i18n("*|All Files");
 	enableButtonApply(type != board);
 
 	previewType = type;
@@ -202,12 +200,9 @@ void Preview::load() {
 
 void Preview::drawPreview()
 {
-	QString tile = "pics/" + Prefs::tileSet();
-	QString back = "pics/" + Prefs::background();
-	QString layout = "pics/" + Prefs::layout();
-	tile = locate("appdata", tile);
-	back = locate("appdata", back);
-	layout = locate("appdata", layout);
+	QString tile = Prefs::tileSet();
+	QString back = Prefs::background();
+	QString layout = Prefs::layout();
 	
 	switch (previewType)
 	{
@@ -263,10 +258,6 @@ void Preview::drawPreview()
 					back = locate("appdata", back);
 					layout = locate("appdata", layout);
 					
-					//parseFile(tilesetRaw, tile);
-					//parseFile(backRaw, back);
-					//parseFile(layoutRaw, layout);
-					
 					in.close();
 					
 					themeBack=back;
@@ -285,20 +276,6 @@ void Preview::paintEvent( QPaintEvent*  ){
   drawFrame->repaint(false);
 }
 
-void Preview::parseFile(const QString &in, QString &out) {
-	//QString prefix = locate("appdata", QString("pics/") +in);
-	QString prefix = locate("appdata", "pics/default.tileset");
-	QFileInfo f( prefix );
-	prefix = f.dirPath();
-
-	// remove any trailing \n
-	QString tmp;
-	tmp.replace(QRegExp("\n"), QString::null);
-	tmp.replace(QRegExp(":"), prefix + '/');
-
-	out = tmp;
-}
-
 // the user selected ok, or apply. This method passes the changes
 // across to the game widget and if necessary forces a board redraw
 // (unnecessary on layout changes since it only effects the next game)
@@ -405,12 +382,9 @@ void Preview::renderTiles(const QString &file, const QString &layout) {
 
 // this really does not belong here. It will be fixed in v1.1 onwards
 void Preview::saveTheme() {
-    QString tile = "pics/" + Prefs::tileSet();
-    QString back = "pics/" + Prefs::background();
-    QString layout = "pics/" + Prefs::layout();
-    tile = locate("appdata", tile);
-    back = locate("appdata", back);
-    layout = locate("appdata", layout);
+    QString tile = Prefs::tileSet();
+    QString back = Prefs::background();
+    QString layout = Prefs::layout();
     
     QString with = ":";
     // we want to replace any path in the default store
@@ -443,11 +417,11 @@ void Preview::saveTheme() {
         return;
     if (f.exists()) {
         // if it already exists, querie the user for replacement
-        int res=KMessageBox::warningYesNo(this,
+        int res=KMessageBox::warningContinueCancel(this,
                         i18n("A file with that name "
                                            "already exists. Do you "
-                                           "wish to overwrite it?"));
-        if (res != KMessageBox::Yes)
+                                           "wish to overwrite it?"),QString::null,i18n("Overwrite"));
+        if (res != KMessageBox::Continue)
                 return ;
     }
     FILE *outFile = fopen( QFile::encodeName(url.path()), "w" );
