@@ -1,6 +1,6 @@
 /*
 
-    $Id: kmahjongg.cpp 401425 2005-03-28 21:52:58Z aacid $
+    $Id: kmahjongg.cpp 465369 2005-09-29 14:33:08Z mueller $
 
     kmahjongg, the classic mahjongg game for KDE project
 
@@ -21,7 +21,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -407,7 +407,7 @@ void KMahjongg::showTileNumber( int iMaximum, int iCurrent, int iLeft )
 {
     // Hmm... seems iCurrent is the number of remaining tiles, not removed ...
     //QString szBuffer = i18n("Removed: %1/%2").arg(iCurrent).arg(iMaximum);
-    QString szBuffer = i18n("Removed: %1/%2  Pairs left: %3").arg(iMaximum-iCurrent).arg(iMaximum).arg(iLeft);
+    QString szBuffer = i18n("Removed: %1/%2  Combinations left: %3").arg(iMaximum-iCurrent).arg(iMaximum).arg(iLeft);
     tilesLeftLabel->setText(szBuffer);
 
     // Update here since undo allow is effected by demo mode
