@@ -14,6 +14,12 @@
 #include "state.h"
 #include "card.h"
 #include "fcs_enums.h"
+#include "app_str.h"
+
+#ifdef DMALLOC
+#include "dmalloc.h"
+#endif
+
 
 #ifndef min
 #define min(a,b) ((a)<(b)?(a):(b))
@@ -22,56 +28,15 @@
 
 #ifdef DEBUG_STATES
 
-fcs_card_t fcs_empty_card = {0,0};
+fcs_card_t freecell_solver_empty_card = {0,0};
 
 #elif defined(COMPACT_STATES) || defined (INDIRECT_STACK_STATES)
 
-fcs_card_t fcs_empty_card = 0;
+fcs_card_t freecell_solver_empty_card = (fcs_card_t)0;
 
 #endif
 
-#ifdef INDIRECT_STACK_STATES
-void fcs_clean_state(
-    fcs_state_with_locations_t * state
-    )
-{
-    int s;
-    for(s=0;s<MAX_NUM_STACKS;s++)
-    {
-        if (state->s.stacks[s] != NULL)
-        {
-            free(state->s.stacks[s]);
-        }
-    }
-}
-#endif
-#ifdef INDIRECT_STACK_STATES
-
-void fcs_duplicate_state_proto(
-    fcs_state_with_locations_t * dest,
-    fcs_state_with_locations_t * src
-    )
-{
-    int a;
-    int stack_len;
-    
-    *dest = *src;
-    for(a=0;a<MAX_NUM_STACKS;a++)
-    {
-        if (src->s.stacks[a] != NULL)
-        {
-            stack_len = src->s.stacks[a][0];
-            dest->s.stacks[a] = malloc(stack_len+13);
-            memset(dest->s.stacks[a], '\0', stack_len+13);
-            memcpy(dest->s.stacks[a], src->s.stacks[a], stack_len+1);
-        }
-    }
-}
-
-#endif
-
-
-int fcs_card_compare(const void * card1, const void * card2)
+static int fcs_card_compare(const void * card1, const void * card2)
 {
     const fcs_card_t * c1 = (const fcs_card_t *)card1;
     const fcs_card_t * c2 = (const fcs_card_t *)card2;
@@ -97,20 +62,20 @@ int fcs_card_compare(const void * card1, const void * card2)
         else
         {
             return 0;
-        }            
+        }
     }
 }
 
 #ifdef DEBUG_STATES
-int fcs_stack_compare(const void * s1, const void * s2)
+static int fcs_stack_compare(const void * s1, const void * s2)
 {
     fcs_card_t card1 = ((const fc_stack_t *)s1)->cards[0];
-    fcs_card_t card2 = ((const fc_stack_t *)s2)->cards[0]; 
+    fcs_card_t card2 = ((const fc_stack_t *)s2)->cards[0];
 
     return fcs_card_compare(&card1, &card2);
 }
 #elif defined(COMPACT_STATES)
-int fcs_stack_compare(const void * s1, const void * s2)
+static int fcs_stack_compare(const void * s1, const void * s2)
 {
     fcs_card_t card1 = ((fcs_card_t*)s1)[1];
     fcs_card_t card2 = ((fcs_card_t*)s2)[1];
@@ -120,15 +85,17 @@ int fcs_stack_compare(const void * s1, const void * s2)
 #elif defined(INDIRECT_STACK_STATES)
 
 
-int fcs_stack_compare_for_stack_sort(const void * s1, const void * s2)
+#if MAX_NUM_DECKS == 1
+static int fcs_stack_compare_for_stack_sort(const void * s1, const void * s2)
 {
     fcs_card_t card1 = ((fcs_card_t*)s1)[1];
     fcs_card_t card2 = ((fcs_card_t*)s2)[1];
 
     return fcs_card_compare(&card1, &card2);
 }
+#endif
 
-int fcs_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
+int freecell_solver_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
 {
     const fcs_card_t * s1 = (const fcs_card_t *)v_s1;
     const fcs_card_t * s2 = (const fcs_card_t *)v_s2;
@@ -137,7 +104,7 @@ int fcs_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
     int a, ret;
 
     min_len = min(s1[0], s2[0]);
-    
+
     for(a=0;a<min_len;a++)
     {
         ret = fcs_card_compare(s1+a+1,s2+a+1);
@@ -148,7 +115,7 @@ int fcs_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
     }
     /*
      * The reason I do the stack length comparisons after the card-by-card
-     * comparison is to maintain correspondence with 
+     * comparison is to maintain correspondence with
      * fcs_stack_compare_for_stack_sort, and with the one card comparison
      * of the other state representation mechanisms.
      * */
@@ -161,7 +128,7 @@ int fcs_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
         return 1;
     }
     else
-    {   
+    {
         return 0;
     }
 }
@@ -169,15 +136,15 @@ int fcs_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
 #endif
 
 #ifdef FCS_WITH_TALONS
-int fcs_talon_compare_with_context(const void * p1, const void * p2, fcs_compare_context_t context)
+static int fcs_talon_compare_with_context(const void * p1, const void * p2, fcs_compare_context_t context)
 {
     fcs_card_t * t1 = (fcs_card_t *)p1;
     fcs_card_t * t2 = (fcs_card_t *)p2;
-    
+
     if (t1[0] < t2[0])
     {
         return -1;
-    }    
+    }
     else if (t1[0] > t2[0])
     {
         return 1;
@@ -190,12 +157,12 @@ int fcs_talon_compare_with_context(const void * p1, const void * p2, fcs_compare
 #endif
 
 #ifdef DEBUG_STATES
-void fcs_canonize_state(fcs_state_with_locations_t * state, int freecells_num, int stacks_num)
+void freecell_solver_canonize_state(fcs_state_with_locations_t * state, int freecells_num, int stacks_num)
 {
     int b,c;
 
     fc_stack_t temp_stack;
-    fcs_card_t temp_freecell;    
+    fcs_card_t temp_freecell;
     int temp_loc;
 
     /* Insertion-sort the stacks */
@@ -203,9 +170,9 @@ void fcs_canonize_state(fcs_state_with_locations_t * state, int freecells_num, i
     {
         c = b;
         while(
-            (c>0) && 
+            (c>0) &&
             (fcs_stack_compare(
-                &(state->s.stacks[c]), 
+                &(state->s.stacks[c]),
                 &(state->s.stacks[c-1])
                 ) < 0)
             )
@@ -221,7 +188,7 @@ void fcs_canonize_state(fcs_state_with_locations_t * state, int freecells_num, i
             c--;
         }
     }
-            
+
     /* Insertion sort the freecells */
 
     for(b=1;b<freecells_num;b++)
@@ -245,14 +212,14 @@ void fcs_canonize_state(fcs_state_with_locations_t * state, int freecells_num, i
 
             c--;
         }
-    }   
+    }
 }
 
 #elif defined(COMPACT_STATES)
 
-void fcs_canonize_state(
-    fcs_state_with_locations_t * state, 
-    int freecells_num, 
+void freecell_solver_canonize_state(
+    fcs_state_with_locations_t * state,
+    int freecells_num,
     int stacks_num)
 {
     int b,c;
@@ -280,7 +247,7 @@ void fcs_canonize_state(
 
             temp_loc = state->stack_locs[c];
             state->stack_locs[c] = state->stack_locs[c-1];
-            state->stack_locs[c-1] = temp_loc;            
+            state->stack_locs[c-1] = temp_loc;
 
             c--;
         }
@@ -313,7 +280,7 @@ void fcs_canonize_state(
     }
 }
 #elif defined(INDIRECT_STACK_STATES)
-void fcs_canonize_state(
+void freecell_solver_canonize_state(
     fcs_state_with_locations_t * state,
     int freecells_num,
     int stacks_num)
@@ -328,17 +295,17 @@ void fcs_canonize_state(
     {
         c = b;
         while(
-            (c>0) && 
+            (c>0) &&
             (
 #if MAX_NUM_DECKS > 1
-                fcs_stack_compare_for_comparison
+                freecell_solver_stack_compare_for_comparison
 #else
                 fcs_stack_compare_for_stack_sort
 #endif
                 (
-                    (state->s.stacks[c]), 
+                    (state->s.stacks[c]),
                     (state->s.stacks[c-1])
-                ) 
+                )
                 < 0
             )
         )
@@ -354,7 +321,7 @@ void fcs_canonize_state(
             c--;
         }
     }
-            
+
     /* Insertion sort the freecells */
 
     for(b=1;b<freecells_num;b++)
@@ -378,12 +345,18 @@ void fcs_canonize_state(
 
             c--;
         }
-    }   
+    }
 }
-    
+
 #endif
 
-void fcs_state_init(fcs_state_with_locations_t * state, int stacks_num)
+static void fcs_state_init(
+    fcs_state_with_locations_t * state, 
+    int stacks_num
+#ifdef INDIRECT_STACK_STATES
+    ,char * indirect_stacks_buffer
+#endif
+    )
 {
     int a;
     memset((void*)&(state->s), 0, sizeof(fcs_state_t));
@@ -394,14 +367,14 @@ void fcs_state_init(fcs_state_with_locations_t * state, int stacks_num)
 #ifdef INDIRECT_STACK_STATES
     for(a=0;a<stacks_num;a++)
     {
-        state->s.stacks[a] = malloc(MAX_NUM_DECKS*52+1);
+        state->s.stacks[a] = &indirect_stacks_buffer[a << 7];
         memset(state->s.stacks[a], '\0', MAX_NUM_DECKS*52+1);
     }
     for(;a<MAX_NUM_STACKS;a++)
     {
         state->s.stacks[a] = NULL;
     }
-#endif    
+#endif
     for(a=0;a<MAX_NUM_FREECELLS;a++)
     {
         state->fc_locs[a] = a;
@@ -410,32 +383,32 @@ void fcs_state_init(fcs_state_with_locations_t * state, int stacks_num)
 
 
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_INDIRECT)
-int fcs_state_compare(const void * s1, const void * s2)
+int freecell_solver_state_compare(const void * s1, const void * s2)
 {
     return memcmp(s1,s2,sizeof(fcs_state_t));
 }
 
-int fcs_state_compare_equal(const void * s1, const void * s2)
+int freecell_solver_state_compare_equal(const void * s1, const void * s2)
 {
     return (!memcmp(s1,s2,sizeof(fcs_state_t)));
 }
 
 
-int fcs_state_compare_with_context(
-    const void * s1, 
-    const void * s2, 
+int freecell_solver_state_compare_with_context(
+    const void * s1,
+    const void * s2,
     fcs_compare_context_t context
     )
 {
     return memcmp(s1,s2,sizeof(fcs_state_t));
 }
 #else
-int fcs_state_compare_indirect(const void * s1, const void * s2)
+int freecell_solver_state_compare_indirect(const void * s1, const void * s2)
 {
     return memcmp(*(fcs_state_with_locations_t * *)s1, *(fcs_state_with_locations_t * *)s2, sizeof(fcs_state_t));
 }
 
-int fcs_state_compare_indirect_with_context(const void * s1, const void * s2, void * context)
+int freecell_solver_state_compare_indirect_with_context(const void * s1, const void * s2, void * context)
 {
     return memcmp(*(fcs_state_with_locations_t * *)s1, *(fcs_state_with_locations_t * *)s2, sizeof(fcs_state_t));
 }
@@ -450,14 +423,17 @@ static const char * const num_redeals_prefixes[] = { "Num-Redeals:", "Readels-Nu
 #define strncasecmp(a,b,c) (strnicmp((a),(b),(c)))
 #endif
 
-fcs_state_with_locations_t fcs_initial_user_state_to_c(
-    const char * string, 
+fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
+    const char * string,
     int freecells_num,
     int stacks_num,
     int decks_num
-#ifdef FCS_WITH_TALONS    
+#ifdef FCS_WITH_TALONS
     ,int talon_type
-#endif    
+#endif
+#ifdef INDIRECT_STACK_STATES
+    , char * indirect_stacks_buffer
+#endif
     )
 {
     fcs_state_with_locations_t ret_with_locations;
@@ -466,13 +442,19 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
     const char * str;
     fcs_card_t card;
     int first_line;
-    
+
     int prefix_found;
     const char * const * prefixes;
     int i;
     int decks_index[4];
 
-    fcs_state_init(&ret_with_locations, stacks_num);
+    fcs_state_init(
+        &ret_with_locations, 
+        stacks_num
+#ifdef INDIRECT_STACK_STATES        
+        , indirect_stacks_buffer  
+#endif
+        );
     str = string;
 
     first_line = 1;
@@ -485,7 +467,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
         fcs_klondike_talon_num_redeals_left(ret) = -1;
     }
 #endif
-    
+
     for(s=0;s<stacks_num;s++)
     {
         /* Move to the next stack */
@@ -510,7 +492,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                 break;
             }
         }
-        
+
         if (prefix_found)
         {
             for(c=0;c<freecells_num;c++)
@@ -522,9 +504,9 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                 if (c!=0)
                 {
                     while(
-                            ((*str) != ' ') && 
-                            ((*str) != '\t') && 
-                            ((*str) != '\n') && 
+                            ((*str) != ' ') &&
+                            ((*str) != '\t') &&
+                            ((*str) != '\n') &&
                             ((*str) != '\r')
                          )
                     {
@@ -563,7 +545,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
             s--;
             continue;
         }
-        
+
         prefixes = foundations_prefixes;
         prefix_found = 0;
         for(i=0;prefixes[i][0] != '\0'; i++)
@@ -575,7 +557,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                 break;
             }
         }
-    
+
         if (prefix_found)
         {
             int d;
@@ -584,7 +566,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
             {
                 fcs_set_foundation(ret, d, 0);
             }
-            
+
             for(d=0;d<4;d++)
             {
                 decks_index[d] = 0;
@@ -601,9 +583,9 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                     str++;
                 c = fcs_u2p_card_number(str);
                 while (
-                        (*str != ' ') && 
-                        (*str != '\t') && 
-                        (*str != '\n') && 
+                        (*str != ' ') &&
+                        (*str != '\t') &&
+                        (*str != '\n') &&
                         (*str != '\r')
                       )
                 {
@@ -633,7 +615,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                 break;
             }
         }
-    
+
         if (prefix_found)
         {
             /* Input the Talon */
@@ -649,9 +631,9 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                 if (c!=0)
                 {
                     while(
-                        ((*str) != ' ') && 
-                        ((*str) != '\t') && 
-                        ((*str) != '\n') && 
+                        ((*str) != ' ') &&
+                        ((*str) != '\t') &&
+                        ((*str) != '\n') &&
                         ((*str) != '\r')
                     )
                     {
@@ -662,19 +644,19 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                         break;
                     }
                 }
-                
+
                 while ((*str == ' ') || (*str == '\t'))
                 {
                     str++;
                 }
-                
+
                 if ((*str == '\n') || (*str == '\r'))
                 {
                     break;
                 }
-                
+
                 card = fcs_card_user2perl(str);
-                
+
                 fcs_put_card_in_talon(ret, c+(talon_type==FCS_TALON_KLONDIKE), card);
             }
             fcs_talon_len(ret) = c;
@@ -682,7 +664,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
             if (talon_type == FCS_TALON_KLONDIKE)
             {
                 int talon_len;
-                
+
                 talon_len = fcs_talon_len(ret);
                 fcs_klondike_talon_len(ret) = talon_len;
                 fcs_klondike_talon_stack_pos(ret) = -1;
@@ -718,8 +700,8 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
                 num_redeals = atoi(str);
                 if (talon_type == FCS_TALON_KLONDIKE)
                 {
-                    fcs_klondike_talon_num_redeals_left(ret) = 
-                        (num_redeals < 0) ? 
+                    fcs_klondike_talon_num_redeals_left(ret) =
+                        (num_redeals < 0) ?
                             (-1) :
                             ((num_redeals > 127) ? 127 : num_redeals)
                                 ;
@@ -729,16 +711,16 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
             continue;
         }
 #endif
-        
+
         for(c=0 ; c < MAX_NUM_CARDS_IN_A_STACK ; c++)
         {
             /* Move to the next card */
             if (c!=0)
             {
                 while(
-                    ((*str) != ' ') && 
-                    ((*str) != '\t') && 
-                    ((*str) != '\n') && 
+                    ((*str) != ' ') &&
+                    ((*str) != '\t') &&
+                    ((*str) != '\n') &&
                     ((*str) != '\r')
                 )
                 {
@@ -761,7 +743,7 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
             card = fcs_card_user2perl(str);
 
             fcs_push_card_into_stack(ret, s, card);
-        } 
+        }
     }
 
     return ret_with_locations;
@@ -769,8 +751,8 @@ fcs_state_with_locations_t fcs_initial_user_state_to_c(
 
 #undef ret
 
-int fcs_check_state_validity(
-    fcs_state_with_locations_t * state_with_locations, 
+int freecell_solver_check_state_validity(
+    fcs_state_with_locations_t * state_with_locations,
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -803,7 +785,7 @@ int fcs_check_state_validity(
             cards[d%4][c]++;
         }
     }
-    
+
     /* Mark the cards in the freecells */
     for(f=0;f<freecells_num;f++)
     {
@@ -814,7 +796,7 @@ int fcs_check_state_validity(
                 [fcs_freecell_card_num(*state, f)] ++;
         }
     }
-    
+
     /* Mark the cards in the stacks */
     for(s=0;s<stacks_num;s++)
     {
@@ -836,8 +818,8 @@ int fcs_check_state_validity(
     /* Mark the cards in the (gypsy) talon */
     if ((talon_type == FCS_TALON_GYPSY) || (talon_type == FCS_TALON_KLONDIKE))
     {
-        for(c = ((talon_type == FCS_TALON_GYPSY)?fcs_talon_pos(*state):1) ; 
-            c < ((talon_type==FCS_TALON_GYPSY) ? fcs_talon_len(*state) : (fcs_klondike_talon_len(*state)+1)) ; 
+        for(c = ((talon_type == FCS_TALON_GYPSY)?fcs_talon_pos(*state):1) ;
+            c < ((talon_type==FCS_TALON_GYPSY) ? fcs_talon_len(*state) : (fcs_klondike_talon_len(*state)+1)) ;
             c++)
         {
             if (fcs_get_talon_card(*state,c) != fcs_empty_card)
@@ -849,9 +831,9 @@ int fcs_check_state_validity(
         }
     }
 #endif
-    
+
     /* Now check if there are extra or missing cards */
-    
+
     for(d=0;d<4;d++)
     {
         for(c=1;c<=13;c++)
@@ -865,19 +847,19 @@ int fcs_check_state_validity(
             }
         }
     }
-    
+
     return 0;
 }
 
 #undef state
 
 
-char * fcs_state_as_string(
-    fcs_state_with_locations_t * state_with_locations, 
-    int freecells_num, 
-    int stacks_num, 
+char * freecell_solver_state_as_string(
+    fcs_state_with_locations_t * state_with_locations,
+    int freecells_num,
+    int stacks_num,
     int decks_num,
-    int parseable_output, 
+    int parseable_output,
     int canonized_order_output,
     int display_10_as_t
     )
@@ -887,10 +869,11 @@ char * fcs_state_as_string(
     int a, card_num_is_null, b;
     int max_num_cards, s, card_num, len;
 
-    char string[8000];
+    /* char string[8000]; */
     char str2[128], str3[128], * str2_ptr, * str3_ptr;
 
-    char * str;
+    freecell_solver_append_string_t * app_str;
+    /* char * str; */
 
     int stack_locs[MAX_NUM_STACKS];
     int freecell_locs[MAX_NUM_FREECELLS];
@@ -919,12 +902,12 @@ char * fcs_state_as_string(
             freecell_locs[(int)(state_with_locations->fc_locs[a])] = a;
         }
     }
-    
+
     for(a=0;a<decks_num*4;a++)
     {
-        fcs_p2u_card_number( 
-            fcs_foundation_value(*state, a), 
-            decks[a], 
+        fcs_p2u_card_number(
+            fcs_foundation_value(*state, a),
+            decks[a],
             &card_num_is_null,
             display_10_as_t,
             0
@@ -933,7 +916,8 @@ char * fcs_state_as_string(
             decks[a][0] = '0';
     }
 
-    str = string;
+    /* str = string; */
+    app_str = freecell_solver_append_string_alloc(512);
 
     if(!parseable_output)
     {
@@ -943,10 +927,10 @@ char * fcs_state_as_string(
             str3_ptr = str3;
             for(b=0;b<min(freecells_num-a*4, 4);b++)
             {
-                str2_ptr += sprintf(str2_ptr, "%3s ", 
+                str2_ptr += sprintf(str2_ptr, "%3s ",
                     fcs_card_perl2user(
                         fcs_freecell_card(
-                            *state, 
+                            *state,
                             freecell_locs[a*4+b]
                         ),
                         freecell,
@@ -957,38 +941,55 @@ char * fcs_state_as_string(
             }
             if (a < decks_num)
             {
-                str += sprintf(str, "%-16s        H-%1s S-%1s D-%1s C-%1s\n",
+                freecell_solver_append_string_sprintf(
+                    app_str,
+                    "%-16s        H-%1s C-%1s D-%1s S-%1s\n",
                     str2,
                     decks[a*4],
                     decks[a*4+1],
                     decks[a*4+2],
-                    decks[a*4+3]);                    
+                    decks[a*4+3]
+                    );
             }
             else
             {
-                str += sprintf(str, "%s\n", str2);
+                freecell_solver_append_string_sprintf(
+                    app_str,
+                    "%s\n", str2
+                    );
             }
-            str += sprintf(str, "%s\n", str3);
+            freecell_solver_append_string_sprintf(
+                app_str,
+                "%s\n", str3
+                );
         }
         for(;a<decks_num;a++)
         {
-            str += sprintf(str, "%-16s        H-%1s S-%1s D-%1s C-%1s\n",
-                    "",
-                    decks[a*4],
-                    decks[a*4+1],
-                    decks[a*4+2],
-                    decks[a*4+3]);    
+            freecell_solver_append_string_sprintf(
+                app_str,
+                "%-16s        H-%1s C-%1s D-%1s S-%1s\n",
+                "",
+                decks[a*4],
+                decks[a*4+1],
+                decks[a*4+2],
+                decks[a*4+3]
+                );
         }
-        *(str++) = '\n';
-        *(str++) = '\n';
-        *str = '\0';
+        freecell_solver_append_string_sprintf(
+            app_str,
+            "%s",
+            "\n\n"
+            );
 
         for(s=0;s<stacks_num;s++)
         {
-            str += sprintf(str, " -- ");
+            freecell_solver_append_string_sprintf(app_str, "%s", " -- ");
         }
-        *(str++) = '\n';
-        *str = '\0';
+        freecell_solver_append_string_sprintf(
+            app_str,
+            "%s",
+            "\n"
+            );
 
         max_num_cards = 0;
         for(s=0;s<stacks_num;s++)
@@ -1005,106 +1006,93 @@ char * fcs_state_as_string(
             {
                 if (card_num >= fcs_stack_len(*state, stack_locs[s]))
                 {
-                    strcpy(str, "    ");
-                    str += strlen(str);
+                    freecell_solver_append_string_sprintf(
+                        app_str,
+                        "    "
+                        );
                 }
                 else
                 {
-                    str += sprintf(
-                        str, 
-                        "%3s ", 
-                        fcs_card_perl2user( 
+                    freecell_solver_append_string_sprintf(
+                        app_str,
+                        "%3s ",
+                        fcs_card_perl2user(
                             fcs_stack_card(
-                                *state, 
-                                stack_locs[s], 
-                                card_num), 
+                                *state,
+                                stack_locs[s],
+                                card_num),
                             stack_card_,
                             display_10_as_t
                             )
-                        );                   
-
+                        );
                 }
             }
-            strcpy(str, "\n");
-            str += strlen(str);
+            freecell_solver_append_string_sprintf(app_str, "%s", "\n");
         }
     }
     else
     {
-#if 0        
-        str += sprintf(str,
-            "Foundations: H-%1s S-%1s D-%1s C-%1s\n"
-            "Freecells: ",
-            decks[0],
-            decks[1],
-            decks[2],
-            decks[3]);
-#endif
-        str += sprintf(str, "Foundations: ");
+        freecell_solver_append_string_sprintf(app_str, "%s", "Foundations: ");
         for(a=0;a<decks_num;a++)
         {
-            str += sprintf(str,
-                "H-%s S-%s D-%s C-%s ",
+            freecell_solver_append_string_sprintf(
+                app_str,
+                "H-%s C-%s D-%s S-%s ",
                 decks[a*4],
                 decks[a*4+1],
                 decks[a*4+2],
                 decks[a*4+3]
                 );
         }
-        str += sprintf(str, "\nFreecells: ");
-        
+
+        freecell_solver_append_string_sprintf(app_str, "%s", "\nFreecells: ");
+
         for(a=0;a<freecells_num;a++)
         {
-            str += sprintf(
-                str, 
-                "%3s", 
+            freecell_solver_append_string_sprintf(
+                app_str,
+                "%3s",
                 fcs_card_perl2user(
                     fcs_freecell_card(
-                        *state, 
+                        *state,
                         freecell_locs[a]
-                    ), 
+                    ),
                     freecell,
                     display_10_as_t
                 )
             );
             if (a < freecells_num-1)
             {
-                *(str++) = ' ';
-                *str = '\0';
+                freecell_solver_append_string_sprintf(app_str, "%s", " ");
             }
         }
-        *(str++) = '\n';
-        *str = '\0';
+        freecell_solver_append_string_sprintf(app_str, "%s", "\n");
 
         for(s=0;s<stacks_num;s++)
         {
-            strcpy(str, ": ");
-            str += strlen(str);
+            freecell_solver_append_string_sprintf(app_str, "%s", ": ");
 
             len = fcs_stack_len(*state, stack_locs[s]);
             for(card_num=0;card_num<len;card_num++)
             {
                 fcs_card_perl2user(
                     fcs_stack_card(
-                        *state, 
-                        stack_locs[s], 
+                        *state,
+                        stack_locs[s],
                         card_num
-                    ), 
+                    ),
                     stack_card_,
                     display_10_as_t
                 );
-                strcpy(str, stack_card_);
-                str += strlen(str);
+                freecell_solver_append_string_sprintf(app_str, "%s", stack_card_);
                 if (card_num < len-1)
                 {
-                    strcpy(str, " ");
-                    str += strlen(str);
+                    freecell_solver_append_string_sprintf(app_str, "%s", " ");
                 }
             }
-            strcpy(str,"\n");
-            str += strlen(str);
+            freecell_solver_append_string_sprintf(app_str, "%s", "\n");
         }
     }
 
-    return strdup(string);
+    return freecell_solver_append_string_finalize(app_str);
 }
