@@ -67,7 +67,7 @@ bool Yukon::isGameLost() const {
          continue;
          }
 
-      if(store[i]->top()->value() == Card::Ace ||
+      if(store[i]->top()->rank() == Card::Ace ||
          ! store[i]->top()->isFaceUp())
          return false;
 
@@ -77,12 +77,12 @@ bool Yukon::isGameLost() const {
          if( !c->isFaceUp() )
             break;
 
-         if(freeStore > 0 && indexi > 0 && c->value() == Card::King)
+         if(freeStore > 0 && indexi > 0 && c->rank() == Card::King)
             return false;
 
          for(j=0; j < 4;j++){
             if(!target[j]->isEmpty() &&
-               c->value()-1 == target[j]->top()->value() &&
+               c->rank()-1 == target[j]->top()->rank() &&
                c->suit() == target[j]->top()->suit())
                return false;
          }
@@ -90,7 +90,7 @@ bool Yukon::isGameLost() const {
          for(j=1; j < 7; j++){
             k=(i+j) % 7;
             if( !store[k]->isEmpty() ) {
-               if(c->value()+1 == store[k]->top()->value() &&
+               if(c->rank()+1 == store[k]->top()->rank() &&
                   (c->isRed() != store[k]->top()->isRed())){
 
                   if(indexi ==  0)
@@ -99,15 +99,15 @@ bool Yukon::isGameLost() const {
                      cNewTop=store[i]->at(indexi-1);
                      if(!cNewTop->isFaceUp())
                         return false;
-                     if(cNewTop->value() == Card::Ace)
+                     if(cNewTop->rank() == Card::Ace)
                         return false;
-                     if(cNewTop->value() != store[k]->top()->value() ||
+                     if(cNewTop->rank() != store[k]->top()->rank() ||
                         cNewTop->isRed() != store[k]->top()->isRed())
                         return false;
 
                      for(l=0; l < 4;l++){
                         if(!target[l]->isEmpty() &&
-                           cNewTop->value()-1 == target[l]->top()->value() &&
+                           cNewTop->rank()-1 == target[l]->top()->rank() &&
                            cNewTop->suit() == target[l]->top()->suit())
                            return false;
                      }
