@@ -11,7 +11,7 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 
@@ -177,7 +177,7 @@ bool PokerWindow::queryClose()
     return true;
 
   // Only ask if the game is changed in some way.
-  switch(KMessageBox::warningYesNoCancel(this, i18n("Do you want to save this game?"))) {
+  switch(KMessageBox::warningYesNoCancel(this, i18n("Do you want to save this game?"), QString::null, KStdGuiItem::save(), KStdGuiItem::dontSave())) {
   case KMessageBox::Yes :
     m_kpok->saveGame();
     return true;
