@@ -38,8 +38,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with KREVERSI; see the file COPYING.  If not, write to
- * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  *******************************************************************
  */
@@ -109,33 +109,35 @@
 class Game {
 public:
   Game();
+  ~Game();
   
   void   newGame();
   
-  Color  color(uint x, uint y) const;
-  uint   score(Color color)    const;
+  Color  color(uint x, uint y) const  { return m_position.color(x, y);  }
+  uint   score(Color color)    const  { return m_position.score(color); }
   Move   lastMove()            const;
   Move   move(uint moveNo)     const;
 
-  bool   moveIsLegal(Move &move) const;
+  bool   moveIsLegal(SimpleMove &move)     const;
   bool   moveIsPossible(Color color) const;
-  bool   moveIsAtAllPossible() const;
-  bool   makeMove(Move &move);
-  bool   takeBackMove();
+  bool   moveIsAtAllPossible()       const;
+  bool   doMove(Move &move);
+  bool   doMove(SimpleMove &move);
+  bool   undoMove();
 
-  uint   moveNumber() const  { return m_moveNumber;        }
-  Color  toMove()     const  { return m_position.toMove(); }
+  const Position &position() const { return m_position; }
+  uint   moveNumber() const        { return m_moveNumber;        }
+  Color  toMove()     const        { return m_position.toMove(); }
 
   // Reversi specific methods
   bool   squareModified(uint x, uint y) const;
   bool   wasTurned(uint x, uint y)      const;
 
-private:
-  Move      m_moves[64];
+protected:
+  Move      m_moves[60];
   Position  m_position;		// The current position in the game
-  Position  m_lastPosition;	// The last position in the game
-                                // So that we can compare and see turned pieces
   uint      m_moveNumber;
 };
 
+
 #endif
