@@ -13,12 +13,10 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
-#include <config.h>
-
 #include "kexthighscore_internal.h"
 
 #include <pwd.h>
@@ -36,6 +34,7 @@
 #include <kmdcodec.h>
 #include <kdebug.h>
 
+#include "config.h"
 #include "kexthighscore.h"
 #include "kexthighscore_gui.h"
 #include "kemailsettings.h"
@@ -512,7 +511,7 @@ void PlayerInfos::removeKey()
 //-----------------------------------------------------------------------------
 ManagerPrivate::ManagerPrivate(uint nbGameTypes, Manager &m)
     : manager(m), showStatistics(false), showDrawGames(false),
-      trackLostGames(false), trackDrawGames(false),
+      trackLostGames(false), trackDrawGames(false), 
       showMode(Manager::ShowForHigherScore),
       _first(true), _nbGameTypes(nbGameTypes), _gameType(0)
 {}
